@@ -33,10 +33,12 @@ from bumble.hci import HCI_Constant
 # -----------------------------------------------------------------------------
 # Constants
 # -----------------------------------------------------------------------------
-GG_GATTLINK_SERVICE_UUID                          = 'ABBAFF00-E56A-484C-B832-8B17CF6CBFE8'
-GG_GATTLINK_RX_CHARACTERISTIC_UUID                = 'ABBAFF01-E56A-484C-B832-8B17CF6CBFE8'
-GG_GATTLINK_TX_CHARACTERISTIC_UUID                = 'ABBAFF02-E56A-484C-B832-8B17CF6CBFE8'
-GG_GATTLINK_L2CAP_CHANNEL_PSM_CHARACTERISTIC_UUID = 'ABBAFF03-E56A-484C-B832-8B17CF6CBFE8'
+GG_GATTLINK_SERVICE_UUID = 'ABBAFF00-E56A-484C-B832-8B17CF6CBFE8'
+GG_GATTLINK_RX_CHARACTERISTIC_UUID = 'ABBAFF01-E56A-484C-B832-8B17CF6CBFE8'
+GG_GATTLINK_TX_CHARACTERISTIC_UUID = 'ABBAFF02-E56A-484C-B832-8B17CF6CBFE8'
+GG_GATTLINK_L2CAP_CHANNEL_PSM_CHARACTERISTIC_UUID = (
+    'ABBAFF03-E56A-484C-B832-8B17CF6CBFE8'
+)
 
 GG_PREFERRED_MTU = 256
 
@@ -44,8 +46,8 @@ GG_PREFERRED_MTU = 256
 # -----------------------------------------------------------------------------
 class GattlinkL2capEndpoint:
     def __init__(self):
-        self.l2cap_channel     = None
-        self.l2cap_packet      = b''
+        self.l2cap_channel = None
+        self.l2cap_packet = b''
         self.l2cap_packet_size = 0
 
     # Called when an L2CAP SDU has been received
@@ -71,12 +73,12 @@ class GattlinkL2capEndpoint:
 class GattlinkHubBridge(GattlinkL2capEndpoint, Device.Listener):
     def __init__(self, device, peer_address):
         super().__init__()
-        self.device                   = device
-        self.peer_address             = peer_address
-        self.peer                     = None
-        self.tx_socket                = None
-        self.rx_characteristic        = None
-        self.tx_characteristic        = None
+        self.device = device
+        self.peer_address = peer_address
+        self.peer = None
+        self.tx_socket = None
+        self.rx_characteristic = None
+        self.tx_characteristic = None
         self.l2cap_psm_characteristic = None
 
         device.listener = self
@@ -97,6 +99,7 @@ class GattlinkHubBridge(GattlinkL2capEndpoint, Device.Listener):
             print(color(f'!!! Connection failed: {error}', 'red'))
 
     @AsyncRunner.run_in_task()
+    # pylint: disable=invalid-overridden-method
     async def on_connection(self, connection):
         print(f'=== Connected to {connection}')
         self.peer = Peer(connection)
@@ -127,7 +130,9 @@ class GattlinkHubBridge(GattlinkL2capEndpoint, Device.Listener):
                 self.rx_characteristic = characteristic
             elif characteristic.uuid == GG_GATTLINK_TX_CHARACTERISTIC_UUID:
                 self.tx_characteristic = characteristic
-            elif characteristic.uuid == GG_GATTLINK_L2CAP_CHANNEL_PSM_CHARACTERISTIC_UUID:
+            elif (
+                characteristic.uuid == GG_GATTLINK_L2CAP_CHANNEL_PSM_CHARACTERISTIC_UUID
+            ):
                 self.l2cap_psm_characteristic = characteristic
         print('RX:', self.rx_characteristic)
         print('TX:', self.tx_characteristic)
@@ -135,7 +140,9 @@ class GattlinkHubBridge(GattlinkL2capEndpoint, Device.Listener):
 
         if self.l2cap_psm_characteristic:
             # Subscribe to and then read the PSM value
-            await self.peer.subscribe(self.l2cap_psm_characteristic, self.on_l2cap_psm_received)
+            await self.peer.subscribe(
+                self.l2cap_psm_characteristic, self.on_l2cap_psm_received
+            )
             psm_bytes = await self.peer.read_value(self.l2cap_psm_characteristic)
             psm = struct.unpack('<H', psm_bytes)[0]
             await self.connect_l2cap(psm)
@@ -150,7 +157,13 @@ class GattlinkHubBridge(GattlinkL2capEndpoint, Device.Listener):
         print(color(f'!!! Connection failed: {error}'))
 
     def on_disconnection(self, reason):
-        print(color(f'!!! Disconnected from {self.peer}, reason={HCI_Constant.error_name(reason)}', 'red'))
+        print(
+            color(
+                f'!!! Disconnected from {self.peer}, '
+                f'reason={HCI_Constant.error_name(reason)}',
+                'red',
+            )
+        )
         self.tx_characteristic = None
         self.rx_characteristic = None
         self.peer = None
@@ -178,7 +191,7 @@ class GattlinkHubBridge(GattlinkL2capEndpoint, Device.Listener):
         pass
 
     # Called by asyncio when a UDP datagram is received
-    def datagram_received(self, data, address):
+    def datagram_received(self, data, _address):
         print(color(f'<<< [UDP]: {len(data)} bytes', 'green'))
 
         if self.l2cap_channel:
@@ -193,11 +206,12 @@ class GattlinkHubBridge(GattlinkL2capEndpoint, Device.Listener):
 class GattlinkNodeBridge(GattlinkL2capEndpoint, Device.Listener):
     def __init__(self, device):
         super().__init__()
-        self.device            = device
-        self.peer              = None
-        self.tx_socket         = None
-        self.tx_subscriber     = None
+        self.device = device
+        self.peer = None
+        self.tx_socket = None
+        self.tx_subscriber = None
         self.rx_characteristic = None
+        self.transport = None
 
         # Register as a listener
         device.listener = self
@@ -212,35 +226,37 @@ class GattlinkNodeBridge(GattlinkL2capEndpoint, Device.Listener):
             GG_GATTLINK_RX_CHARACTERISTIC_UUID,
             Characteristic.WRITE_WITHOUT_RESPONSE,
             Characteristic.WRITEABLE,
-            CharacteristicValue(write=self.on_rx_write)
+            CharacteristicValue(write=self.on_rx_write),
         )
         self.tx_characteristic = Characteristic(
             GG_GATTLINK_TX_CHARACTERISTIC_UUID,
             Characteristic.NOTIFY,
-            Characteristic.READABLE
+            Characteristic.READABLE,
         )
         self.tx_characteristic.on('subscription', self.on_tx_subscription)
         self.psm_characteristic = Characteristic(
             GG_GATTLINK_L2CAP_CHANNEL_PSM_CHARACTERISTIC_UUID,
             Characteristic.READ | Characteristic.NOTIFY,
             Characteristic.READABLE,
-            bytes([psm, 0])
+            bytes([psm, 0]),
         )
         gattlink_service = Service(
             GG_GATTLINK_SERVICE_UUID,
-            [
-                self.rx_characteristic,
-                self.tx_characteristic,
-                self.psm_characteristic
-            ]
+            [self.rx_characteristic, self.tx_characteristic, self.psm_characteristic],
         )
         device.add_services([gattlink_service])
         device.advertising_data = bytes(
-            AdvertisingData([
-                (AdvertisingData.COMPLETE_LOCAL_NAME, bytes('Bumble GG', 'utf-8')),
-                (AdvertisingData.INCOMPLETE_LIST_OF_128_BIT_SERVICE_CLASS_UUIDS,
-                 bytes(reversed(bytes.fromhex('ABBAFF00E56A484CB8328B17CF6CBFE8'))))
-            ])
+            AdvertisingData(
+                [
+                    (AdvertisingData.COMPLETE_LOCAL_NAME, bytes('Bumble GG', 'utf-8')),
+                    (
+                        AdvertisingData.INCOMPLETE_LIST_OF_128_BIT_SERVICE_CLASS_UUIDS,
+                        bytes(
+                            reversed(bytes.fromhex('ABBAFF00E56A484CB8328B17CF6CBFE8'))
+                        ),
+                    ),
+                ]
+            )
         )
 
     async def start(self):
@@ -251,7 +267,7 @@ class GattlinkNodeBridge(GattlinkL2capEndpoint, Device.Listener):
         self.transport = transport
 
     # Called by asyncio when a UDP datagram is received
-    def datagram_received(self, data, address):
+    def datagram_received(self, data, _address):
         print(color(f'<<< [UDP]: {len(data)} bytes', 'green'))
 
         if self.l2cap_channel:
@@ -263,14 +279,17 @@ class GattlinkNodeBridge(GattlinkL2capEndpoint, Device.Listener):
             asyncio.create_task(self.device.notify_subscribers(self.tx_characteristic))
 
     # Called when a write to the RX characteristic has been received
-    def on_rx_write(self, connection, data):
+    def on_rx_write(self, _connection, data):
         print(color(f'<<< [GATT RX]: {len(data)} bytes', 'cyan'))
         print(color('>>> [UDP]', 'magenta'))
         self.tx_socket.sendto(data)
 
     # Called when the subscription to the TX characteristic has changed
     def on_tx_subscription(self, peer, enabled):
-        print(f'### [GATT TX] subscription from {peer}: {"enabled" if enabled else "disabled"}')
+        print(
+            f'### [GATT TX] subscription from {peer}: '
+            f'{"enabled" if enabled else "disabled"}'
+        )
         if enabled:
             self.tx_subscriber = peer
         else:
@@ -290,7 +309,15 @@ class GattlinkNodeBridge(GattlinkL2capEndpoint, Device.Listener):
 
 
 # -----------------------------------------------------------------------------
-async def run(hci_transport, device_address, role_or_peer_address, send_host, send_port, receive_host, receive_port):
+async def run(
+    hci_transport,
+    device_address,
+    role_or_peer_address,
+    send_host,
+    send_port,
+    receive_host,
+    receive_port,
+):
     print('<<< connecting to HCI...')
     async with await open_transport_or_link(hci_transport) as (hci_source, hci_sink):
         print('<<< connected')
@@ -307,14 +334,14 @@ async def run(hci_transport, device_address, role_or_peer_address, send_host, se
         # Create a UDP to RX bridge (receive from UDP, send to RX)
         loop = asyncio.get_running_loop()
         await loop.create_datagram_endpoint(
-            lambda: bridge,
-            local_addr=(receive_host, receive_port)
+            lambda: bridge, local_addr=(receive_host, receive_port)
         )
 
         # Create a UDP to TX bridge (receive from TX, send to UDP)
         bridge.tx_socket, _ = await loop.create_datagram_endpoint(
+            # pylint: disable-next=unnecessary-lambda
             lambda: asyncio.DatagramProtocol(),
-            remote_addr=(send_host, send_port)
+            remote_addr=(send_host, send_port),
         )
 
         await device.power_on()
@@ -328,15 +355,43 @@ async def run(hci_transport, device_address, role_or_peer_address, send_host, se
 @click.argument('hci_transport')
 @click.argument('device_address')
 @click.argument('role_or_peer_address')
-@click.option('-sh', '--send-host', type=str, default='127.0.0.1', help='UDP host to send to')
+@click.option(
+    '-sh', '--send-host', type=str, default='127.0.0.1', help='UDP host to send to'
+)
 @click.option('-sp', '--send-port', type=int, default=9001, help='UDP port to send to')
-@click.option('-rh', '--receive-host', type=str, default='127.0.0.1', help='UDP host to receive on')
-@click.option('-rp', '--receive-port', type=int, default=9000, help='UDP port to receive on')
-def main(hci_transport, device_address, role_or_peer_address, send_host, send_port, receive_host, receive_port):
-    asyncio.run(run(hci_transport, device_address, role_or_peer_address, send_host, send_port, receive_host, receive_port))
+@click.option(
+    '-rh',
+    '--receive-host',
+    type=str,
+    default='127.0.0.1',
+    help='UDP host to receive on',
+)
+@click.option(
+    '-rp', '--receive-port', type=int, default=9000, help='UDP port to receive on'
+)
+def main(
+    hci_transport,
+    device_address,
+    role_or_peer_address,
+    send_host,
+    send_port,
+    receive_host,
+    receive_port,
+):
+    asyncio.run(
+        run(
+            hci_transport,
+            device_address,
+            role_or_peer_address,
+            send_host,
+            send_port,
+            receive_host,
+            receive_port,
+        )
+    )
 
 
 # -----------------------------------------------------------------------------
-logging.basicConfig(level = os.environ.get('BUMBLE_LOGLEVEL', 'WARNING').upper())
+logging.basicConfig(level=os.environ.get('BUMBLE_LOGLEVEL', 'WARNING').upper())
 if __name__ == '__main__':
     main()
