@@ -16,21 +16,21 @@
 # Imports
 # -----------------------------------------------------------------------------
 import asyncio
+import logging
 import sys
 import os
+from bumble.device import AdvertisingType, Device
+from bumble.hci import Address
 
-from bumble.hci import *
-from bumble.controller import *
-from bumble.device import *
-from bumble.transport import *
-from bumble.host import *
 from bumble.transport import open_transport_or_link
 
 
 # -----------------------------------------------------------------------------
 async def main():
     if len(sys.argv) < 3:
-        print('Usage: run_advertiser.py <config-file> <transport-spec> [type] [address]')
+        print(
+            'Usage: run_advertiser.py <config-file> <transport-spec> [type] [address]'
+        )
         print('example: run_advertiser.py device1.json usb:0')
         return
 
@@ -56,6 +56,7 @@ async def main():
         await device.start_advertising(advertising_type=advertising_type, target=target)
         await hci_source.wait_for_termination()
 
+
 # -----------------------------------------------------------------------------
-logging.basicConfig(level = os.environ.get('BUMBLE_LOGLEVEL', 'DEBUG').upper())
+logging.basicConfig(level=os.environ.get('BUMBLE_LOGLEVEL', 'DEBUG').upper())
 asyncio.run(main())
